import javax.comm.*;
import java.io.*;
import java.util.*;
import java.sql.*;
import java.net.*;

class WCListener{

	private SerialParameters parameters;
	private SerialConnection connection;
	public MultilineClientFrame client;
	public UDPserver server;
	public UDPlistener udplistener;
	public boolean running = true;
	private boolean runAsServer = false;
	private boolean runBroadcast = true;
	private boolean runXmlLogger = false;
	private boolean popups = true;
	private boolean runLog = true;
    private FileOutputStream log;
	public xmlLogger xlog;
	private String logName = "/WhozzCalling.log"; // default log file
	public int udpPort = 6699; // default udp broadcast/recieve port
	public String udpAddress = "224.0.0.0";
	private Properties props = null;

	public static void main(String[] args) {
		if ((args.length > 0) 
		    && (args[0].equals("-h") 
		    || args[0].equals("-help"))) {
		    System.out.println("usage: java WCListener [options]");
		    System.out.println("---");
		    System.out.println("Options:");
		    System.out.println("   -c                   Run as a network CLIENT");
		    System.out.println("   -s                   Run as a network SERVER (default is CLIENT)");
		    System.out.println("   -n  {IP_ADDRESS}     Set network broadcast/receive ADDRESS");
		    System.out.println("   -u  {PORT}           Set network broadcast/receive PORT (default is 6699)");
		    System.out.println("   -p  {COM}            Set COM port address (default is COM1 or /dev/ttyS0)");
		    System.out.println("   -t                   Turn OFF network broadcast (default is ON)");
		    System.out.println("   -o                   Turn OFF popups (overrides form control)");
		    System.out.println("   -l                   Turn OFF logging");
		    System.out.println("   -x                   Use XML format logging");
		    System.out.println("   -f  {LOGFILE}        Sets the LOGFILE location");
		    System.out.println("\n\n");
		    System.out.println("System is running: " + System.getProperty("os.name"));
		    System.exit(1);
		}
		
		WCListener wclistener = new WCListener(args);
    }

    private void parseArgs(String[] args) {
		int i;
		
		if (args.length < 1) {
		    return;
		}else{
			for (i=0; i<args.length; i++){
					/*
					if(args[i].equals("-s")){
						File f = new File(args[i+1]);
					
						if (!f.exists()) {
						    f = new File(System.getProperty("user.dir")
								 + System.getProperty("path.separator")
								 + args[0]);
						}
					
						if (f.exists()) {
						    try {
							FileInputStream fis = new FileInputStream(f);
							props = new Properties();
							props.load(fis);
							fis.close();
							loadParams();
						    } catch (IOException e) {
						    }
						} 					
					}else */
					if(args[i].equals("-c")){ // -c runs this as a network CLIENT
						runAsServer = false;
						System.err.println("SETTING PROGRAM FOR CLIENT MODE");
					}else if(args[i].equals("-s")){ // -s runs this as a network SERVER
						runAsServer = true;
						System.err.println("SETTING PROGRAM FOR SERVER MODE");
					}else if(args[i].equals("-t")){ // -t argument turns OFF UDP Broadcasting
						runBroadcast = false;
					}else if(args[i].equals("-p")){ // -p sets the COM PORT
						parameters.setPortName(args[i+1]);
					}else if(args[i].equals("-u")){ // -u sets the UDP BROADCAST/LISTEN PORT
						udpPort = Integer.parseInt(args[i+1]);
					}else if(args[i].equals("-n")){ // -n sets the UDP BROADCAST/LISTEN NETWORK ADDRESS
						udpAddress = args[i+1];
					}else if(args[i].equals("-o")){ // -o turns OFF pop-ups
						popups = false;
					}else if(args[i].equals("-l")){ // -l turns OFF logging
						runLog = false;
					}else if(args[i].equals("-x")){ // -x uses xmlLogger
						runXmlLogger = true;
					}else if(args[i].equals("-f")){ // -f sets the logfile name
						logName = args[i+1];
					}
			}
		}
   }
	
	private void loadParams() {
		parameters.setPortName(props.getProperty("portName"));
		parameters.setBaudRate(props.getProperty("baudRate"));
		parameters.setFlowControlIn(props.getProperty("flowControlIn"));
		parameters.setFlowControlOut(props.getProperty("flowControlOut"));
		parameters.setParity(props.getProperty("parity"));
		parameters.setDatabits(props.getProperty("databits"));
		parameters.setStopbits(props.getProperty("stopbits"));
	
    }

    public WCListener(String[] args){
		parameters = new SerialParameters();
		if (System.getProperty("os.name").startsWith("L")){
			parameters.setPortName("/dev/ttyS0");
		}else{
			parameters.setPortName("COM1");
		}
		// get any arguments passed, and update settings using args
		parseArgs(args);
		
		// create the GUI
		client = new MultilineClientFrame(this, popups, "application");
		client.show();

		if (runAsServer){ // start in server mode
			connection = new SerialConnection(this, parameters);
		
			try{
				connection.openConnection();
				System.err.println("STARTING IN SERVER MODE:");
				client.updateWindow("STARTING IN SERVER MODE", null);
			}catch(SerialConnectionException e){
				System.err.println("Error Opening Serial Port Connection: " + e);
			}
			if (runBroadcast){
				server = new UDPserver(udpPort, udpAddress);
				System.err.println("  starting broadcast server: " + udpAddress + ":" + udpPort);
			}		
		}else{ // start in client mode
			System.err.println("STARTING IN CLIENT MODE:");
			client.updateWindow("STARTING IN CLIENT MODE", null);
			try{
				udplistener = new UDPlistener(this, InetAddress.getByName(udpAddress), udpPort);
				udplistener.setDaemon(true);
				udplistener.start();
				System.err.println("starting network listener as: " + udpAddress + ":" + udpPort);
				client.updateWindow("starting network listener as: " + udpAddress + ":" + udpPort, null);
			}catch(UnknownHostException e){
				// ???
			}		
		}
		
		if (runLog){ // log file components
			if (runXmlLogger){ // xmlLogger - generates XML data for each call
		    		xlog = new xmlLogger(logName);
		    		System.err.println("XML logfile started: " + logName);
			}else{ // plain text file
				try {
		    		File logFile = new File(logName);
		    		log = new FileOutputStream(logFile, true);
		    		java.util.Date now = new java.util.Date();
		    		String logstartup = new String("WhozzCalling Listener Started: " + now.toString() + "\n");
		    		byte[] startlog = logstartup.getBytes();
		    		log.write(startlog);
		    		System.err.println("logfile started: " + logFile.getAbsolutePath());
				}catch (IOException e) {
		    		// Oops, no log file
		    		System.err.println("ERROR OPENING LOGFILE:");
				}
			}
		}
    }
	
	public synchronized void shutdown(boolean exit){
		if (exit){
			System.err.println("\nSHUTTING DOWN...");
			running = false;
			try{
				xlog.close();
				log.close();
				server.shutdown();
				connection.closeConnection();
				client.dispose();
			}catch(IOException closeEx){
				// could not close the log file
			}catch(NullPointerException shutdownNPE){
				// argh
			}
			System.err.println("SHUT DOWN COMPLETE");
			System.exit(0);
		}
	}
		
	public synchronized void parseData(String strIn){
		String dLn = "";
		
		try{
			dLn = strIn.substring(0, 2);
		}catch(java.lang.StringIndexOutOfBoundsException e){
		
		}
		
		if (dLn.equals("V8")){ 							// WC box has sent a Version String
			client.updateWindow(strIn, null);
		}else{
			String[] logdata = client.callData(strIn);
			if (runAsServer && runBroadcast){
				server.sendData(strIn);
			}
			if (runLog){
				if (runXmlLogger){
					String[] logtypes = {"line", "io", "start", "duration", "checksum", "rings", "date", "time", "phone", "name"};
			    	xlog.log(logtypes, logdata);
				}else{
					try{
						String message = strIn + "\n";
						byte[] messagebytes = message.getBytes();
				    	log.write(messagebytes);
				    }catch (IOException F){
				    	// Could not write to log file
				    	System.err.println("Error writing to logfile");
				    }
				}
	    	}
	    }
	}

		

}
